#!/bin/ksh
TSLM_CONF_FILE="/etc/tslm.cfg"
find_value_tslm_cfg()
{
    TARGETSECT=$1
    TARGETITEM=$2 
    awk 'BEGIN {FS="=";flag=0} /^\[.*\].*$/ {flag=0} /^\['"$TARGETSECT"'\].*$/ {flag=1} flag==1 && $1=="'${TARGETITEM}'" {print $2}' $TSLM_CONF_FILE
}

read_tslm_cfg() # for testing only
{
    TARGETSECT=$1
    if [ -e "$TSLM_CONF_FILE" -a  "$TARGETSECT" != "" ]; then
        echo "Reading $TSLM_CONF_FILE..."

        for TARGETITEM in "TSLM_GROUP" "TSLM_USER" "TSLM_USERDIR" "TSLM_BASEDIR" "DB_ADM_NAME" "DB_ADM_GRP_NAME" "DB_ADM_DIR" "SQLLIB_PATH"
        do
            line=`find_value_tslm_cfg $TARGETSECT $TARGETITEM`
            if [[ $line != "" ]]; then
                eval $TARGETITEM="$line"
            else
                case "$target" in
		    "TSLM_GROUP") TSLM_GROUP="ermm" ;;
		    "TSLM_USER") TSLM_USER="ermm" ;;
		    "TSLM_USERDIR") TSLM_USERDIR="/home/ermm" ;;
		    "TSLM_BASEDIR") TSLM_BASEDIR="/opt/IBM/ermm" ;;
                    "DB_ADM_NAME") DB_ADM_NAME="db2inst1" ;;
                    "DB_ADM_GRP_NAME") DB_ADM_GRP_NAME="db2grp1" ;;
                    "DB_ADM_DIR") DB_ADM_DIR="/home/db2inst1" ;;
                    "SQLLIB_PATH") SQLLIB_PATH="/home/db2inst1/sqllib/bin" ;;
                esac
            fi
        done
    else
	TSLM_GROUP="ermm"
	TSLM_USER="ermm"
	TSLM_USERDIR="/home/ermm"
	TSLM_BASEDIR="/opt/IBM/ermm"
        DB_ADM_NAME="db2inst1"
        DB_ADM_GRP_NAME="db2grp1"
        DB_ADM_DIR="/home/db2inst1"
        SQLLIB_PATH="/home/db2inst1/sqllib/bin"
    fi
}

TSLM_NO=""
copyright="yes"


while getopts "C:" Option
do
    case $Option in
	"c" ) copyright="no";;
	"C" ) TSLM_NO="$OPTARG";;
    esac
done

read_tslm_cfg $TSLM_NO
TSLM_PROFILE2=$TSLM_USERDIR/tslmprofile
TSLM_PROFILE3=$DB_ADM_DIR/tslmprofile

#
# adjust $1 variable by processed option
#
shift $(($OPTIND - 1))

if [ $copyright == "yes" ] ; then
    echo "IBM Tape System Library Manager"
    echo "Licensed Materials - Property of IBM"
    echo "(C) Copyright IBM Corp. 2004, 2018 All Rights Reserved"
fi

#
# Check if group exists
#
if (! grep "$TSLM_GROUP:" /etc/group > /dev/null); then
    echo "No defined group: $TSLM_GROUP..."
    exit 1
fi

#
# Check if user exists
#
if (! grep "$TSLM_USER:" /etc/passwd > /dev/null); then
    echo "No defined user: $TSLM_USER..."
    exit 1
fi

#
# Check if installation directory exists
#
if ! [ -d $TSLM_BASEDIR ]; then
    echo "No directory: $TSLM_BASEDIR..."
    exit 1
fi

echo "Creating $TSLM_PROFILE2..."
#
# Create new profile and set path to the installation
#
LINE=`awk 'BEGIN {FS="=";flag=0} /^\[.*\].*$/ {flag=0} /^\['"$TSLM_NO"'\].*$/ {flag=1} flag==1 && !/^\[TSLM.+\].*$/ {print $0}' $TSLM_CONF_FILE`
echo "$LINE" > $TSLM_PROFILE2

chown $TSLM_USER $TSLM_PROFILE2
chgrp $TSLM_GROUP $TSLM_PROFILE2

echo "Creating $TSLM_PROFILE3..."
echo "$LINE" > $TSLM_PROFILE3
chown $DB_ADM_NAME $TSLM_PROFILE3
chgrp $DB_ADM_GRP_NAME $TSLM_PROFILE3

