#!/usr/bin/perl
#
#
# - create clones of TSLM instances
# - erase clones of TSLM instances
#
#
use warnings;
use Getopt::Std;
use File::Path;
use FindBin;

our $TSLMCFG="/etc/tslm.cfg";

# TSLM config repository
our %tslmConfig;

# parameters of tslm.cfg
# temporary values
our $TSLM_NO="";
our $TSLM_USER="";
our $TSLM_GROUP="";
our $TSLM_BASEDIR="";
our $TSLM_USERDIR="";
our $DB_ADM_NAME="";
our $DB_ADM_GRP_NAME="";
our $DB_ADM_DIR="";
our $SQLLIB_PATH="";

# parameters for clone
our $BINDIR=$FindBin::Bin;
our $CLONELST="$BINDIR/TSLMClone.lst";
our %PKGSCOPE=(OPD=>"",PKG=>"",CHK=>"");
our $SKIP_PKGSCOPE="F";
my  %opt;
our $SRCDIR="";
our $DESTDIR="";
our %DIRSCOPE=(OPD=>"",DIR=>"",MODE=>"",USER=>"",GROUP=>"");
our $SKIP_DIRSCOPE="F";
our $DESTUSER="";
our $DESTGROUP="";
our $DESTHOMEDIR="";
our $DEST_DB_ADM_DIR="";
our $DEST_SQLLIB_PATH="";
our $DEST_DB_ADM_NAME="";
our $DEST_DB_ADM_GRP_NAME="";

our $SCSLOGDIR="/var/log";
our $T_CREATEUSER="/opt/IBM/ermm/install/createUser";
our $MNGMODE="";

our $TARGET="";
# usage
sub usage{
    print "Usage: manageTSLMClone.pl -C -T target -s srcdir -d destdir -u user -g group -h homedir -a dbadmindir -l sqllibpath -i db2user -j db2group\n";
    print "     : manageTSLMClone.pl -C [-f] -T \"TSLM_0\" -d destdir -u user -g group -h homedir -a dbadmindir -l sqllibpath -i db2user -j db2group\n";
    print "     : manageTSLMClone.pl -C [-f]\n";
    print "     : manageTSLMClone.pl -c|-e|-E -T target\n";
    print "     : manageTSLMClone.pl -q [-T target]\n";
    print "  -C          - create section in TSLM configuration file\n";
    print "  -E          - erase section on TSLM configuration file\n";
    print "  -c          - create instance\n";
    print "  -e          - erase instance\n";
    print "  -q          - show parameters\n";
    print "  -f          - execute forcibly\n";
    print "  target      - target(destination) incident name (\"TSLM_1\" for example)\n";
    print "  srcdir      - source directory\n";
    print "  destdir     - destination directory\n";
    print "  user        - username of new instance\n";
    print "  group       - groupname of new instance\n";
    print "  homedir     - home directory of new instance\n";
    print "  dbadmindir  - admin directory of DB2 instance\n";
    print "  sqllibpath  - sqplib directory of DB2 instance\n";
    print "  db2user     - user name of DB2 instance\n";
    print "  db2group    - db2 group of DB2 instance\n";
}

# exit
sub exit_w_msg{
    my $rc=shift;
    my $msg=shift;

    if ($rc==1){
	warn "Error - $msg doesn't exist\n";
	exit $rc;
    }elsif ($rc==2){
	warn "Error - invalid target instance name:$msg \n";
	exit $rc;
    }elsif ($rc==3){
	warn "Error - $msg directory deosn't exist\n";
	exit $rc;
    }elsif ($rc==4){
	warn "Error - failed to make $msg\n";
	exit $rc;
    }elsif ($rc==5){
	warn "Error - $msg exists already.\n";
	exit $rc;
    }elsif ($rc==6){
	warn "Error - primary section [$msg] isn't created.\n";
	exit $rc;
    }elsif ($rc==7){
	warn "Error - primary instance [$msg] isn't installed correctly\n";
	exit $rc;
    }elsif ($rc==8){
	warn "Error - instance [$msg] isn't created correctly\n";
	exit $rc;
    }elsif ($rc==9){
	warn "Error - configuration file [$msg] is invalid\n";
	exit $rc;
    }
    exit 1;
}

# validity check for primary section
sub checkPrimarySection{
    
    my $rc=0;
    if( ! isRecord("TSLM_0") ){
	exit_w_msg(6,"TSLM_0");
    }
    
    my $_TSLM_BASEDIR=getValue("TSLM_0","TSLM_BASEDIR");
    my $_TSLM_USER=getValue("TSLM_0","TSLM_USER");
    my $_TSLM_GROUP=getValue("TSLM_0","TSLM_GROUP");
    my $_TSLM_NO=getValue("TSLM_0","TSLM_NO");
    my $_TSLM_USERDIR=getValue("TSLM_0","TSLM_USERDIR");
    my $_DB_ADM_DIR=getValue("TSLM_0","DB_ADM_DIR");
    my $_SQLLIB_PATH=getValue("TSLM_0","SQLLIB_PATH");
    my $_DB_ADM_NAME=getValue("TSLM_0","DB_ADM_NAME");
    my $_DB_ADM_GRP_NAME=getValue("TSLM_0","DB_ADM_GRP_NAME");

    if ( ! -d $_TSLM_BASEDIR ){
	warn "Primary TSLM base directory [$_TSLM_BASEDIR] dosen't exist.\n";
	$rc=1;
    }

    if ( ! -d $_TSLM_USERDIR ){
	warn "Primary TSLM home directory [$_TSLM_USERDIR] doesn't exist.\n";
	$rc=1;
    }

    if ( ! -d $_DB_ADM_DIR ){
	warn "DB2 instance directory for primary TSLM doesn't exist.\n";
	$rc=1;
    }

    if ( ! -d $_SQLLIB_PATH ){
	warn "DB2 SQLLIB directory for primary TSLM doesn't exist.\n";
	$rc=1;
    }

    if ( system("grep $_TSLM_USER /etc/passwd | cut -f1 -d: | grep $_TSLM_USER\$ > /dev/null") ){
	warn "$_TSLM_USER user name for TSLM primary instance is not created.\n"; 
	$rc=1;
    }

    if ( system("grep $_DB_ADM_NAME /etc/passwd | cut -f1 -d: | grep $_DB_ADM_NAME\$ > /dev/null") ){
	warn "$_DB_ADM_NAME DB2 user name for TSLM primary instance is not created.\n"; 
	$rc=1;
    }
    
    if ( system("grep $_TSLM_GROUP /etc/group | cut -f1 -d: | grep $_TSLM_GROUP\$ > /dev/null") ){
	warn "$_TSLM_GROUP group name for TSLM primary instance is not created.\n"; 
	$rc=1;
    }

    if ( system("grep $_DB_ADM_GRP_NAME /etc/group | cut -f1 -d: | grep $_DB_ADM_GRP_NAME\$ > /dev/null") ){
	warn "$_DB_ADM_GRP_NAME DB2 group name for TSLM primary instance is not created.\n"; 
	$rc=1;
    }
    
    if ( $rc ){
	exit_w_msg(7,"TSLM_0");
    }
}

# validity check for specified section
sub checkSection{
    my $SECTION=shift;
    
    my $rc=0;
    if( ! isRecord($SECTION) ){
	exit_w_msg(6,$SECTION);
    }
    
    my $_TSLM_BASEDIR=getValue($SECTION,"TSLM_BASEDIR");
    my $_TSLM_USER=getValue($SECTION,"TSLM_USER");
    my $_TSLM_GROUP=getValue($SECTION,"TSLM_GROUP");
    my $_TSLM_NO=getValue($SECTION,"TSLM_NO");
    my $_TSLM_USERDIR=getValue($SECTION,"TSLM_USERDIR");
    my $_DB_ADM_DIR=getValue($SECTION,"DB_ADM_DIR");
    my $_SQLLIB_PATH=getValue($SECTION,"SQLLIB_PATH");
    my $_DB_ADM_NAME=getValue($SECTION,"DB_ADM_NAME");
    my $_DB_ADM_GRP_NAME=getValue($SECTION,"DB_ADM_GRP_NAME");

    if ( ! -d $_TSLM_BASEDIR ){
	print "Trying to create TSLM base directory [$_TSLM_BASEDIR] for $SECTION.\n";
	if (!mkpath ($_TSLM_BASEDIR)){
	    warn "Failed to create TSLM base directory [$_TSLM_BASEDIR].\n";
	    $rc=1;
	}else{
	    print "$_TSLM_BASEDIR directory created\n";
	}
    }


    if ( ! -d $_DB_ADM_DIR ){
	warn "DB2 instance directory for TSLM $SECTION instance doesn't exist.\n";
	$rc=1;
    }

    if ( ! -d $_SQLLIB_PATH ){
	warn "DB2 SQLLIB directory for TSLM $SECTION instance doesn't exist.\n";
	$rc=1;
    }


    if ( system("grep $_DB_ADM_NAME /etc/passwd | cut -f1 -d: | grep $_DB_ADM_NAME\$ > /dev/null") ){
	warn "$_DB_ADM_NAME DB2 user name for TSLM $SECTION instance is not created.\n"; 
	$rc=1;
    }
    
    if ( system("grep $_DB_ADM_GRP_NAME /etc/group | cut -f1 -d: | grep $_DB_ADM_GRP_NAME\$ > /dev/null") ){
	warn "$_DB_ADM_GRP_NAME DB2 group name for TSLM $SECTION instance is not created.\n"; 
	$rc=1;
    }
    
    if ( $rc ){
	exit_w_msg(8,$SECTION);
    }
}

# clear temporary values
sub clearTSLMtmp{
    $TSLM_NO="";
    $TSLM_USER="";
    $TSLM_GROUP="";
    $TSLM_BASEDIR="";
    $TSLM_USERDIR="";
    $DB_ADM_NAME="";
    $DB_ADM_GRP_NAME="";
    $DB_ADM_DIR="";
    $SQLLIB_PATH="";
}

# create config record and push into repository
sub createRecord{
    my $rec = {
	TSLM_NO=>$TSLM_NO,
	TSLM_USER=>$TSLM_USER,
	TSLM_GROUP=>$TSLM_GROUP,
	TSLM_BASEDIR=>$TSLM_BASEDIR,
	TSLM_USERDIR=>$TSLM_USERDIR,
	DB_ADM_NAME=>$DB_ADM_NAME,
	DB_ADM_GRP_NAME=>$DB_ADM_GRP_NAME,
	DB_ADM_DIR=>$DB_ADM_DIR,
	SQLLIB_PATH=>$SQLLIB_PATH};
    $tslmConfig{$TSLM_NO}=$rec;
}

sub deleteRecord{
    my $index=shift;
    delete($tslmConfig{$index});
}

sub readConfigFile{
    clearTSLMtmp();
    open ( CFG_FILE, "< $TSLMCFG" ) or die ( "Error - Open $TSLMCFG file failure. :$!" );
    while ( my $line = <CFG_FILE> ){
	chomp($line);
	$line =~ s/^\s+//; #remove spaces beginning of line
	if ($line =~ /^\#.*/){
	    print "$line\n";
	}else{
	    my @parm = split(/\s*=\s*/,$line);

	    if ($parm[0]=~/\[TSLM_[0-9]+\]/){
		if($TSLM_NO ne ""){
		    createRecord();
		    clearTSLMtmp();
		}
		$TSLM_NO=$parm[0];
		$TSLM_NO=~s/\[(.+)\]/$1/g;
	    }
	    if( $parm[0] eq "TSLM_USER" ) { if (defined($parm[1])){$TSLM_USER=$parm[1]; }else{$TSLM_USER="";} }
	    if( $parm[0] eq "TSLM_GROUP" )  { if (defined($parm[1])){$TSLM_GROUP=$parm[1]; }else{$TSLM_GROUP="";} }
	    if( $parm[0] eq "TSLM_BASEDIR" ){ if (defined($parm[1])){$TSLM_BASEDIR=$parm[1]; }else{$TSLM_BASEDIR="";} }
	    if( $parm[0] eq "TSLM_USERDIR" ){ if (defined($parm[1])){$TSLM_USERDIR=$parm[1];}else{$TSLM_USERDIR="";} }
	    if( $parm[0] eq "DB_ADM_NAME" ) { if (defined($parm[1])){$DB_ADM_NAME=$parm[1];}else{$DB_ADM_NAME="";} }
	    if( $parm[0] eq "DB_ADM_GRP_NAME" ){ if (defined($parm[1])){$DB_ADM_GRP_NAME=$parm[1];}else{$DB_ADM_GRP_NAME="";} }
	    if( $parm[0] eq "DB_ADM_DIR" ){ if (defined($parm[1])){$DB_ADM_DIR=$parm[1];}else{$DB_ADM_DIR="";} }
	    if( $parm[0] eq "SQLLIB_PATH" ){ if (defined($parm[1])){$SQLLIB_PATH=$parm[1];}else{$SQLLIB_PATH="";} }
	}
    }
    if ($TSLM_NO ne ""){
	createRecord();
	clearTSLMtmp();
    }
    close ( CFG_FILE );
}

sub listRepository{
#    printf("### TSLM CONFIGURATION ###\n");
    foreach my $reckey(sort keys(%tslmConfig)){
	my $rec=$tslmConfig{$reckey};
	printf("[%s]\n",$rec->{TSLM_NO});
	foreach my $param(keys %$rec){
	    printf("\t%s=%s\n",$param,$rec->{$param});
	}
    }
}

sub listRepositoryAcc{
#    printf("### TSLM CONFIGURATION ###\n");
    my $reckey=shift;
    my $rec=$tslmConfig{$reckey};
#    printf("[%s]\n",$rec->{TSLM_NO});
    foreach my $param(keys %$rec){
	printf("%s=%s\n",$param,$rec->{$param});
    }
}

sub writeConfigFile{
    if ( -e $TSLMCFG ){
	if(system("cp -fa $TSLMCFG $TSLMCFG.bak")){
	    warn "Error - failed to backup $TSLMCFG\n";
	    return 1;
	}
    }
    open ( CFG_FILE, "> $TSLMCFG" ) or die ( "Error - Open $TSLMCFG file failure. :$!" );
    foreach my $reckey(sort keys(%tslmConfig)){
	my $rec=$tslmConfig{$reckey};
	print CFG_FILE "[$rec->{TSLM_NO}]\n";
	foreach my $param(keys %$rec){
	    print CFG_FILE "$param=$rec->{$param}\n";
	}
    }
    close ( CFG_FILE );
    return 0;
}

sub isRecord{
    my $index=shift;
    if (defined($tslmConfig{$index})){
	return 1;
    }
    return 0;
}

sub getValue{
    my $index=shift;
    my $param=shift;
    my $value=$tslmConfig{$index}->{$param};
    if (defined($value)){
	return $value;
    }else{
	return "nil";
    }
}

sub setValue{
    my $index=shift;
    my $param=shift;
    my $value=shift;
    if (isRecord($index)){
	$tslmConfig{$index}->{$param}=$value;
	return 0;
    }
    return 1;
}

# check if the user is in the local system
sub isUserName{
    my $userName = shift;
    my $result=system("cat /etc/passwd | cut -f1 -d: | grep $userName\$");
    return $result;
}

# check if the group is in the local system
sub isGroupName{
    my $groupName = shift;
    my $result=system("cat /etc/group | cut -f1 -d: | grep $groupName\$");
    return $result;
}

# get user
sub getUserName{
    my $userName = shift;
    if ($userName eq "root"){
	return "root";
    }
    if ($userName eq "DESTUSER"){
	return $DESTUSER;
    }
    return "ermm";
}

# get group
sub getGroupName{
    my $groupName = shift;
    if ($groupName eq "root"){
	return "root";
    }
    if ($groupName eq "DESTGROUP"){
	return $DESTGROUP;
    }
    return "ermm";
}

# get user home directory
sub getHomeDirectory{
    my $userName = shift;
    open my $rs, "grep $userName /etc/passwd | cut -f6 -d: 2&1 |";
    my @rlist = <$rs>;
    close $rs;
    my $result = join '', @rlist;
    return $result;
}

# pkgBegin
sub pkgBegin{
    my (@parm)=@_;
    %PKGSCOPE=("OPD" => $parm[0], "PKG" => $parm[1], "CHK" => $parm[2]);
    if ( ! -e $SRCDIR."/".$PKGSCOPE{CHK} ){
	warn "the file[".$SRCDIR."/".$PKGSCOPE{CHK}."] doesn't exist. $PKGSCOPE{PKG} is not installed.\n";
	$SKIP_PKGSCOPE="T";
    }
}

# pkgEnd
sub pkgEnd{
    my (@parm)=@_;
    if($PKGSCOPE{PKG} ne $parm[1]){
	warn "the package name is invalid. PKGSCOPE=$PKGSCOPE{PKG}, PKGEND=$parm[1]\n";
    }	
    %PKGSCOPE=(OPD=>"",PKG=>"",CHK=>"");
    $SKIP_PKGSCOPE="F";
}

# dirBegin
sub dirBegin{
    my (@parm)=@_;
    %DIRSCOPE=("OPD"=>$parm[0], "DIR"=>$parm[1], "MODE"=>$parm[2], "USER"=>$parm[3], "GROUP"=>$parm[4]);
    if ($SKIP_PKGSCOPE eq "T") { return; }
    if ( ! -d $SRCDIR."/".$DIRSCOPE{DIR} ){ # check if the soruce directory exists
	warn "the file[".$SRCDIR."/".$DIRSCOPE{DIR}."] doesn't exist. $DIRSCOPE{DIR} is not installed.\n";
	$SKIP_DIRSCOPE="T";
    }
    if ($MNGMODE eq "CREATE_I"){
	if ( ! -d $DESTDIR."/".$DIRSCOPE{DIR} ){# check if the destination directory exists
	    mkpath ($DESTDIR."/".$DIRSCOPE{DIR}) || warn "Error - failed to make $DESTDIR/$DIRSCOPE{DIR}\n";
	}
	if (system("chmod $DIRSCOPE{MODE} $DESTDIR/$DIRSCOPE{DIR}")){
	    warn "Error - failed to change permissions $DESTDIR/$DIRSCOPE{DIR}\n";
	}
	my $USERMODE;
	my $GROUPMODE;
	if ( $DIRSCOPE{USER} eq "root" ){ $USERMODE="root" }
	if ( $DIRSCOPE{USER} eq "USER" ){ $USERMODE=$DESTUSER }
	if ( $DIRSCOPE{GROUP} eq "root" ){ $GROUPMODE="root" }
	if ( $DIRSCOPE{GROUP} eq "GROUP" ){ $GROUPMODE=$DESTGROUP }

	if (system("chown $USERMODE.$GROUPMODE $DESTDIR/$DIRSCOPE{DIR}")){
	    warn "Error - failed to change owner $DESTDIR/$DIRSCOPE{DIR}\n";
	}
    }
}

# dirEnd
sub dirEnd{
    my (@parm)=@_;
    if($DIRSCOPE{DIR} ne $parm[1]){
	warn "the package name is invalid. DIRSCOPE=$DIRSCOPE{DIR}, DIREND=$parm[1]\n";
    }else{
	if( (($SKIP_PKGSCOPE eq "T") && ($parm[2] eq "LOCAL")) || ((($MNGMODE eq "ERASE")||($MNGMODE eq "ERASE_I")) && ($parm[2] eq "LOCAL")) ){
	    system("rmdir $DESTDIR/$DIRSCOPE{DIR} >/dev/null 2>&1");
	}
    }
    %DIRSCOPE=(OPD=>"",DIR=>"",MODE=>"",USER=>"",GROUP=>"");
    $SKIP_DIRSCOPE="F";
}

# mkLnk
sub mkLnk{
    my $DEL=0;
    if ($SKIP_PKGSCOPE eq "T") { $DEL=1; }
    if ($SKIP_DIRSCOPE eq "T") { $DEL=1; }
    if (($MNGMODE eq "ERASE")||($MNGMODE eq "ERASE_I")) { $DEL=1; }
    my (@parm)=@_;
    if ( $DEL ){
	if ( -l $DESTDIR."/".$DIRSCOPE{DIR}."/".$parm[1] ){
	    if (system("rm -f $DESTDIR/$DIRSCOPE{DIR}/$parm[1]")){
		warn "Error - failed to remove unlinked file [$DESTDIR/$DIRSCOPE{DIR}/$parm[1]]\n";
	    }
	}
    }else{
	if ( ! -e $SRCDIR."/".$DIRSCOPE{DIR}."/".$parm[1] ){
	    if ( -l $DESTDIR."/".$DIRSCOPE{DIR}."/".$parm[1] ){
		if (system("rm -f $DESTDIR/$DIRSCOPE{DIR}/$parm[1]")){
		    warn "Error - failed to remove unlinked file [$DESTDIR/$DIRSCOPE{DIR}/$parm[1]]\n";
		}
	    }
	}else{
	    if (system("ln -s -f $SRCDIR/$DIRSCOPE{DIR}/$parm[1] $DESTDIR/$DIRSCOPE{DIR}/$parm[1]")){
		warn "Error - failed to create link file from [$SRCDIR/$DIRSCOPE{DIR}/$parm[1]] to [$DESTDIR/$DIRSCOPE{DIR}/$parm[1]]\n";
	    }
	}
    }
}

# mkCp
sub mkCp{
    my $DEL=0;
    if ($SKIP_PKGSCOPE eq "T") { $DEL=1; }
    if ($SKIP_DIRSCOPE eq "T") { $DEL=1; }
    if (($MNGMODE eq "ERASE")||($MNGMODE eq "ERASE_I")) { $DEL=1; }
    my (@parm)=@_;
    if ( $DEL ){
	if ( -e $DESTDIR."/".$DIRSCOPE{DIR}."/".$parm[1] ){
	    if (system("rm -f $DESTDIR/$DIRSCOPE{DIR}/$parm[1]")){
		warn "Error - failed to remove unlinked file [$DESTDIR/$DIRSCOPE{DIR}/$parm[1]]\n";
	    }
	}
    }else{
	if ( -e $SRCDIR."/".$DIRSCOPE{DIR}."/".$parm[1] ){
	    if (system("cp -a -f $SRCDIR/$DIRSCOPE{DIR}/$parm[1] $DESTDIR/$DIRSCOPE{DIR}/$parm[1]")){
		warn "Error - failed to create link file from [$SRCDIR/$DIRSCOPE{DIR}/$parm[1]] to [$DESTDIR/$DIRSCOPE{DIR}/$parm[1]]\n";
	    }
	}
    }
}

# runPreScript
sub runPreScript{
    if ($SKIP_PKGSCOPE eq "T") { return; }
    my (@parm)=@_;
    if( $parm[1] eq "tslm-system" ){ 
	my $result=system("$SRCDIR/$parm[2] $DESTGROUP");
	if ($result){
	    warn "Error[runPreScript] - failed to execute $parm[2] script.\n";
	}
    }
    if( $parm[1] eq "tslm-mm" ){
	my $result=system("$SRCDIR/$parm[2] $DESTGROUP");
	if ($result){
	    warn "Error[runPreScript] - failed to execute $parm[2] script.\n";
	}
    }
    if( $parm[1] eq "tslm-lm" ){
	my $result=system("$SRCDIR/$parm[2] $DESTGROUP");
	if ($result){
	    warn "Error[runPreScript] - failed to execute $parm[2] script.\n";
	}
    }
    if( $parm[1] eq "tslm-hdm" ){
	my $result=system("$SRCDIR/$parm[2] $DESTGROUP");
	if ($result){
	    warn "Error[runPreScript] - failed to execute $parm[2] script.\n";
	}
    }
    if( $parm[1] eq "tslm-elm" ){
	my $result=system("$SRCDIR/$parm[2] $DESTGROUP");
	if ($result){
	    warn "Error[runPreScript] - failed to execute $parm[2] script.\n";
	}
    }	    
}

# runUnPreScript
sub runUnPreScript{
    my (@parm)=@_;
    if( $parm[1] eq "tslm-mm" ){
	my $result=system("$SRCDIR/$parm[2] $DESTDIR");
	if ($result){
	    warn "Error[runUnPreScript] - failed to execute $parm[2] script.\n";
	}
    }
}

# runUnPostScript
sub runUnPostScript{
    my (@parm)=@_;
    if( $parm[1] eq "tslm-hdm" ){
	my $result=system("$SRCDIR/$parm[2] GHDM_$DESTUSER $DESTDIR");
	if ($result){
	    warn "Error[runUnPostScript] - failed to execute $parm[2] script.\n";
	}
    }
    if( $parm[1] eq "tslm-3494" ){
	my $result=system("$SRCDIR/$parm[2] ERLM_$DESTUSER $DESTDIR");
	if ($result){
	    warn "Error[runUnPostScript] - failed to execute $parm[2] script.\n";
	}
    }
}

# runPostScript
sub runPostScript{

    if ($SKIP_PKGSCOPE eq "T") { return; }
    my (@parm)=@_;

    if( $parm[1] eq "tslm-system" ){ 
	my $result=system("$SRCDIR/$parm[2] $TARGET");
	if ($result){
	    warn "Error[runPostScript] - failed to execute $parm[2] script.\n";
	}
    }
    if( $parm[1] eq "tslm-lm" ){ 
	my $result=system("$SRCDIR/$parm[2] $DESTUSER $SCSLOGDIR");
	if ($result){
	    warn "Error[runPostScript] - failed to execute $parm[2] script.\n";
	}
    }
    if( $parm[1] eq "tslm-hdm" ){
	my $result=system("$SRCDIR/$parm[2] GHDM_$DESTUSER $DESTDIR");
	if ($result){
	    warn "Error[runPostScript] - failed to execute $parm[2] script.\n";
	}
    }
    if( $parm[1] eq "tslm-3494" ){
	my $result=system("$SRCDIR/$parm[2] ERLM_$DESTUSER $DESTDIR");
	if ($result){
	    warn "Error[runPostScript] - failed to execute $parm[2] script.\n";
	}
    }
    if( $parm[1] eq "tslm-cli" ){
	my $result=system("$SRCDIR/$parm[2] $DESTDIR $DESTHOMEDIR $DESTUSER $DESTGROUP");
	if ($result){
	    warn "Error[runPostScript] - failed to execute $parm[2] script.\n";
	}
    }
}

sub verifyDup{
    foreach my $reckey1 (sort keys(%tslmConfig)){
	foreach my $reckey2 (sort keys(%tslmConfig)){
	    my $TSLM_NO_1=$tslmConfig{$reckey1}->{TSLM_NO};
	    my $TSLM_NO_2=$tslmConfig{$reckey2}->{TSLM_NO};
	    my $BASEDIR1=$tslmConfig{$reckey1}->{TSLM_BASEDIR};
	    my $BASEDIR2=$tslmConfig{$reckey2}->{TSLM_BASEDIR};

	    if ( $TSLM_NO_1 eq $TSLM_NO_2 ){ 
		next;
	    }
	    if ( $BASEDIR1 eq $BASEDIR2 ) {
		warn("Error - Duplicate: [%s]:TSLM_BASEDIR=[%s], [%s]:TSLM_BASEDIR=[%s]\n", $TSLM_NO_1, $BASEDIR1, $TSLM_NO_2, $BASEDIR2);
		warn "Check a configuration file:$TSLMCFG\n";
		return 1;
	    }
	}
    }
    return 0;
}

sub setTSLMParameter{
    my $SRC_NO=shift;
    my $TGT_NO=shift;
    if(! isRecord($SRC_NO) ){
	warn "$SRC_NO is not defined. Check $TSLMCFG";
	return 1;
    }
    if(! isRecord($TGT_NO) ){
	warn "$TGT_NO is not defined. Check $TSLMCFG";
	return 1;
    }
    $SRCDIR=getValue($SRC_NO,"TSLM_BASEDIR");
    $DESTDIR=getValue($TGT_NO,"TSLM_BASEDIR");
    $DESTUSER=getValue($TGT_NO,"TSLM_USER");
    $DESTGROUP=getValue($TGT_NO,"TSLM_GROUP");
    $DESTHOMEDIR=getValue($TGT_NO,"TSLM_USERDIR");
    $T_CREATEUSER="$SRCDIR/install/createUser";
    return 0;
}

# general purpose variables
my $rc=0;  #for return code
#
# check command line parameter
#
getopts("s:d:u:g:h:a:l:i:j:ceqCET:f"=>\%opt);
# validity check of parameters

my $num=keys %opt;

if ( ($num==11) # create section
     && exists $opt{C}
     && exists $opt{T}
     && exists $opt{s}
     && exists $opt{d}
     && exists $opt{u}
     && exists $opt{g}
     && exists $opt{h}
     && exists $opt{a}
     && exists $opt{l}
     && exists $opt{i}
     && exists $opt{j} ){
    $MNGMODE="CREATE";
} elsif ( ($num==10) # create primary section
     && exists $opt{C}
     && exists $opt{T}
     && exists $opt{d}
     && exists $opt{u}
     && exists $opt{g}
     && exists $opt{h}
     && exists $opt{a}
     && exists $opt{l}
     && exists $opt{i}
     && exists $opt{j} ){
    $MNGMODE="CREATE_P";
} elsif ( ($num==11) # create primary section
     && exists $opt{C}
     && exists $opt{T}
     && exists $opt{f}
     && exists $opt{d}
     && exists $opt{u}
     && exists $opt{g}
     && exists $opt{h}
     && exists $opt{a}
     && exists $opt{l}
     && exists $opt{i}
     && exists $opt{j} ){
    $MNGMODE="CREATE_PF";
} elsif ( ($num==1) # create primary section with default value
     && exists $opt{C}){
    $MNGMODE="CREATE_D";
} elsif ( ($num==2) # create primary section with default value
     && exists $opt{C}
     && exists $opt{f}){
    $MNGMODE="CREATE_DF";
} elsif ( ($num==2) # create instance
          && exists $opt{c}
	  && exists $opt{T} ){
    $MNGMODE="CREATE_I";
} elsif ( ($num==2) # erase instance
          && exists $opt{e}
	  && exists $opt{T} ){
    $MNGMODE="ERASE_I";
} elsif ( ($num==2) # erace section
	  && exists $opt{E}
	  && exists $opt{T} ){
    $MNGMODE="ERASE";
} elsif ( ($num==2) # query parameters
	  && exists $opt{q}
	  && exists $opt{T} ){
    $MNGMODE="QUERY";
} elsif ( ($num==1) # query all parameters
	  && exists $opt{q} ){
    $MNGMODE="QUERYALL";
} else {
    usage();
    exit -1;
}

# srcdir
if (exists $opt{s}){
    if (defined $opt{s}){
	$SRCDIR=$opt{s};
    }else{
	usage();
	exit_w_msg(1,"'-s' option");
    }
}

# destdir
if (exists $opt{d}){
    if (defined $opt{d}){
	$DESTDIR=$opt{d};
    }else{
	usage();
	exit_w_msg(1,"'-d' option");
    }
}

# user
if (exists $opt{u}){
    if (defined $opt{u}){
	$DESTUSER=$opt{u};
    }else{
	usage();
	exit_w_msg(1,"'-u' option");
    }
}

# group
if (exists $opt{g}){
    if (defined $opt{g}){
	$DESTGROUP=$opt{g};
    }else{
	usage();
	exit_w_msg(1,"'-g' option");
    }
}
    
# homedir
if (exists $opt{h}){
    if (defined $opt{h}){
	    $DESTHOMEDIR=$opt{h};
    }else{
	usage();
	exit_w_msg(1,"'-h' option");
    }
}

# dbadmindir
if (exists $opt{a}){
    if (defined $opt{a}){
	$DEST_DB_ADM_DIR=$opt{a};
    }else{
	usage();
	exit_w_msg(1,"'-a' option");
    }
}

# sqllibpath
if (exists $opt{l}){
    if (defined $opt{l}){
	$DEST_SQLLIB_PATH=$opt{l};
    }else{
	usage();
	exit_w_msg(1,"'-l' option");
    }
}

# db2user
if (exists $opt{i}){
    if (defined $opt{i}){
	$DEST_DB_ADM_NAME=$opt{i};
    }else{
	usage();
	exit_w_msg(1,"'-i' option");
    }
}

# db2group
if (exists $opt{j}){
    if (defined $opt{j}){
	$DEST_DB_ADM_GRP_NAME=$opt{j};
    }else{
	usage();
	exit_w_msg(1,"'-j' option");
    }
}

# target
if (exists $opt{T}){
    if (defined $opt{T}){
	$TARGET=$opt{T};
	if ( $TARGET eq "TSLM_0" ){
	    if ( ($MNGMODE ne "CREATE_P")
		 &&($MNGMODE ne "CREATE_PF") 
		 &&($MNGMODE ne "QUERY")
		 &&($MNGMODE ne "QUERYALL")
		){
		exit_w_msg(2,"$TARGET");
	    }
	} elsif ( ! ($TARGET =~ /^TSLM_[1-9][0-9]*$/) ) {
	    exit_w_msg(2,"$TARGET");
	}
    }else{
	usage();
	exit_w_msg(1,"'-T' option");
    }    
}

if ( ($MNGMODE eq "CREATE_P") || ($MNGMODE eq "CREATE_PF") ){ # create primary section
    if ( $TARGET ne "TSLM_0" ){
	exit_w_msg(2,"$TARGET");
    }
    if ( $MNGMODE eq "CREATE_P" ){
	if ( -e $TSLMCFG ){
	    exit_w_msg(5,"$TSLMCFG");
	}
    }
    clearTSLMtmp();
    $TSLM_NO=$TARGET;
    $TSLM_USER=$DESTUSER;
    $TSLM_GROUP=$DESTGROUP;
    $TSLM_BASEDIR=$DESTDIR;
    $TSLM_USERDIR=$DESTHOMEDIR;
    $DB_ADM_NAME=$DEST_DB_ADM_NAME;
    $DB_ADM_GRP_NAME=$DEST_DB_ADM_GRP_NAME;
    $DB_ADM_DIR=$DEST_DB_ADM_DIR;
    $SQLLIB_PATH=$DEST_SQLLIB_PATH;
    createRecord();
    writeConfigFile();
    exit 0;
}
if ( ($MNGMODE eq "CREATE_D") || ($MNGMODE eq "CREATE_DF") ){ # create primary section with default
    if ( $MNGMODE eq "CREATE_D" ){
	if ( -e $TSLMCFG ){
	    exit_w_msg(5,"$TSLMCFG");
	}
    }
    clearTSLMtmp();
    $TSLM_NO="TSLM_0";
    $TSLM_USER="ermm";
    $TSLM_GROUP="ermm";
    $TSLM_BASEDIR="/opt/IBM/ermm";
    $TSLM_USERDIR="/home/ermm";
    $DB_ADM_NAME="db2inst1";
    $DB_ADM_GRP_NAME="db2grp1";
    $DB_ADM_DIR="/home/db2inst1";
    $SQLLIB_PATH="/home/db2inst1/sqllib/bin";
    createRecord();
    writeConfigFile();
    exit 0;
}

if ( $MNGMODE eq "CREATE" ) {
# read config file
    clearTSLMtmp();
    readConfigFile();
    $TSLM_NO=$TARGET;
    $TSLM_USER=$DESTUSER;
    $TSLM_GROUP=$DESTGROUP;
    $TSLM_BASEDIR=$DESTDIR;
    $TSLM_USERDIR=$DESTHOMEDIR;
    $DB_ADM_NAME=$DEST_DB_ADM_NAME;
    $DB_ADM_GRP_NAME=$DEST_DB_ADM_GRP_NAME;
    $DB_ADM_DIR=$DEST_DB_ADM_DIR;
    $SQLLIB_PATH=$DEST_SQLLIB_PATH;
    createRecord();
    writeConfigFile();
    exit 0;
}

if ( $MNGMODE eq "ERASE" ) {
    if ( $TARGET eq "TSLM_0" ){
	exit_w_msg(2,"$TARGET");
    }
# read config file
    clearTSLMtmp();
    readConfigFile();
    if (isRecord($TARGET)){
	deleteRecord($TARGET);
    }else{
	exit_w_msg(2,"$TARGET");
    }
    writeConfigFile();
    exit 0;
}

if ( $MNGMODE eq "QUERY" ) {
# read config file
    readConfigFile();
    listRepositoryAcc($TARGET);

    exit 0;
}

if ( $MNGMODE eq "QUERYALL" ) {
# read config file
    readConfigFile();
    listRepository();

    exit 0;
}

if ( $MNGMODE eq "CREATE_I" ) {

# read config file
    readConfigFile();
# check if the primary instance exists
    if (! isRecord("TSLM_0") ){
	warn "Primary instance (TSLM_O) is not installed. Please check $TSLMCFG and if the TSLM is installed.\n";
	exit_w_msg(1,"TSLM_0");
    }

# validity check of config file
    if (verifyDup()) {
	exit_w_msg(9,$TSLMCFG);
    }
    
# check source section
    checkPrimarySection();

# set TSLM parameter
    if ( setTSLMParameter("TSLM_0", $TARGET) ){
	exit_w_msg(8,$TARGET);
    }

# check destination section
    checkSection($TARGET);

#
# parse TSLMClone.lst
#
    open ( CLONELST_FILE, "< $CLONELST" ) or die ( "Error - Open $CLONELST file failure. :$!" );
    while ( my $line = <CLONELST_FILE> ){
	chomp($line);
	$line =~ s/^\s+//; #remove spaces beginning of line
	if ($line =~ /^\#.*/){
	    print "$line\n";
	}else{
	    my @parm = split(/,/,$line);
	    if ( defined($parm[0]) ){
		if( $parm[0] eq "PKGBEGIN" ){pkgBegin(@parm);}
		if( $parm[0] eq "PKGEND" ){pkgEnd(@parm);}
		if( $parm[0] eq "DIRBEGIN" ){dirBegin(@parm);}
		if( $parm[0] eq "DIREND" ){dirEnd(@parm);}
		if( $parm[0] eq "LNK" ){mkLnk(@parm);}
		if( $parm[0] eq "CP" ){mkCp(@parm);}
		if( $parm[0] eq "PRESCRIPT" ){runPreScript(@parm);}
		if( $parm[0] eq "POSTSCRIPT" ){runPostScript(@parm);}
	    }
	}
    }
    close ( CLONELST_FILE );
    print "Done\n";
    exit 0;
}

if ( $MNGMODE eq "ERASE_I" ) {
# read config file
    readConfigFile();
# check if the primary instance exists
    if ( $TARGET eq "TSLM_0"){
	warn "Primary instance (TSLM_O) can't be removed. Please use an uninstall script\n";
	exit_w_msg(2,"TSLM_0");
    }

# validity check of config file
    if (verifyDup()) {
	exit_w_msg(9,$TSLMCFG);
    }
    
# set TSLM parameter
    if ( setTSLMParameter("TSLM_0", $TARGET) ){
	exit_w_msg(8,$TARGET);
    }
#
# parse TSLMClone.lst
#
    open ( CLONELST_FILE, "< $CLONELST" ) or die ( "Error - Open $CLONELST file failure. :$!" );
    while ( my $line = <CLONELST_FILE> ){
	chomp($line);
	$line =~ s/^\s+//; #remove spaces beginning of line
	if ($line =~ /^\#.*/){
	    print "$line\n";
	}else{
	    my @parm = split(/,/,$line);
	    if ( defined($parm[0]) ){
		if( $parm[0] eq "PKGBEGIN"){pkgBegin(@parm);}
		if( $parm[0] eq "PKGEND" ){pkgEnd(@parm);}
		if( $parm[0] eq "DIRBEGIN" ){dirBegin(@parm);}
		if( $parm[0] eq "DIREND" ){dirEnd(@parm);}
		if( $parm[0] eq "LNK" ){mkLnk(@parm);}
		if( $parm[0] eq "CP" ){mkCp(@parm);}
		if( $parm[0] eq "UNPRESCRIPT" ){runUnPreScript(@parm);}
		if( $parm[0] eq "UNPOSTSCRIPT" ){runUnPostScript(@parm);}
	    }
	}
    }
    close ( CLONELST_FILE );
    print "Done\n";
    exit 0;
}


exit 0;

