#!/bin/sh

###################################
# MAIN
###################################
if [ $# -ne 2 ]; then
    echo "Error[post3494.sh]: - invalid parameter"
    exit 1
fi
SERVICE_NAME=$1
SERVICE_PATH=$2
SERVICE_FILE=$SERVICE_PATH/client/3494/ermm3494

#
# whatDist()
# DIST value:
# 0: not supported OS
# 1: RHEL5
# 2: RHEL6
# 3: RHEL7
# 4: SLES10-11
# 5: SLES12
# 9: AIX
whatDist()
{
    OS=`uname`
    if [ $OS = "AIX" ]; then # AIX
	DIST=9
    elif [ $OS = "Linux" ]; then # Linux
	if [ -e "/etc/redhat-release" ]; then           # RHEL
	    RHELVER="`cat /etc/redhat-release | sed -e 's/^.*\(6\.\).*$/\1/'`"
	    if [ $RHELVER = "6." ]; then #RHEL6
		DIST=2
	    elif [ $RHELVER = "7." ]; then #RHEL7
		DIST=3
	    else                         #RHEL5
		DIST=1
	    fi
	elif [ -e "/etc/SuSE-release" ]; then           # SuSE
	    SLESVER="`cat /etc/SuSE-release | grep VERSION | tr -d ' ' | cut -f2 -d'='`"
	    if [ $SLESVER = "12" ]; then
		DIST=5                 #SuSE12
	    else
		DIST=4                 #SuSE10-11
	    fi
	else                                            # Others 
	    DIST=0
	fi
    else
	DIST=0
    fi
}

#
# post 3494 for RHEL5
#
post3494_RHEL5()
{
    if (! grep "^$SERVICE_NAME:" /etc/inittab > /dev/null); then
	echo -n "Adding $SERVICE_NAME to /etc/inittab..."
	echo "$SERVICE_NAME:2345:respawn:$SERVICE_FILE -d >/dev/null 2>&1 # TSLM 3494 Emulation" >>/etc/inittab
        echo "done"
        echo -n "Telling init to re-examine /etc/inittab..."
        telinit q
        echo "done"
    fi
}

#
# post 3494 for RHEL6
#
post3494_RHEL6()
{
    echo -n "Creating $SERVICE_NAME.conf in /etc/init..."
    echo "# 3494 - tslm 3494 emulation" > /etc/init/$SERVICE_NAME.conf
    echo "start on stopped rc RUNLEVEL=[2345]" >> /etc/init/$SERVICE_NAME.conf
    echo "stop on starting rc RUNLEVEL=[!2345]" >> /etc/init/$SERVICE_NAME.conf
    echo "respawn" >> /etc/init/$SERVICE_NAME.conf
    echo "exec $SERVICE_FILE -d >/dev/null 2>&1" >> /etc/init/$SERVICE_NAME.conf
    initctl start $SERVICE_NAME
    echo "done"
}

#
# post 3494 for RHEL7
#
post3494_RHEL7()
{
    echo -n "Creating $SERVICE_NAME.service in /etc/systemd/system..."
    echo "# 3494 -tslm 3494 emulation" > /etc/systemd/system/$SERVICE_NAME.service
    echo "[Unit]" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "Description=3494 emulation" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "After=syslog.target" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "[Service]" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "Type=simple" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "ExecStart=/bin/perl $SERVICE_FILE -d >/dev/null 2>&1" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "StandardOutput=syslog" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "StandardError=syslog" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "Restart=always" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "[Install]" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "WantedBy=multi-user.target" >> /etc/systemd/system/$SERVICE_NAME.service
    chmod 777 /etc/systemd/system/$SERVICE_NAME.service
    /usr/bin/systemctl enable $SERVICE_NAME.service
    /usr/bin/systemctl restart $SERVICE_NAME.service
}

#
# post 3494 for SuSE10
#
post3494_SLES10()
{
    if (! grep "^$SERVICE_NAME:" /etc/inittab > /dev/null); then
	echo -n "Adding $SERVICE_NAME to /etc/inittab..."
	echo "$SERVICE_NAME:2345:respawn:$SERVICE_FILE -d >/dev/null 2>&1 # TSLM 3494 Emulation" >>/etc/inittab
        echo "done"
        echo -n "Telling init to re-examine /etc/inittab..."
        telinit q
        echo "done"
    fi
}

#
# post 3494 for SuSE12
#
post3494_SLES12()
{
    echo -n "Creating $SERVICE_NAME.service in /etc/systemd/system..."
    echo "# 3494 -tslm 3494 emulation" > /etc/systemd/system/$SERVICE_NAME.service
    echo "[Unit]" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "Description=3494 emulation" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "After=syslog.target" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "[Service]" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "Type=simple" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "ExecStart=/bin/perl $SERVICE_FILE -d >/dev/null 2>&1" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "StandardOutput=syslog" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "StandardError=syslog" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "Restart=always" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "[Install]" >> /etc/systemd/system/$SERVICE_NAME.service
    echo "WantedBy=multi-user.target" >> /etc/systemd/system/$SERVICE_NAME.service
    chmod 777 /etc/systemd/system/$SERVICE_NAME.service
    /usr/bin/systemctl enable $SERVICE_NAME.service
    /usr/bin/systemctl restart $SERVICE_NAME.service
}

#
# post 3494 for AIX
#
post3494_AIX()
{
    if (! grep "^$SERVICE_NAME:" /etc/inittab > /dev/null); then
        echo -n "Adding $SERVICE_NAME to /etc/inittab..."
        echo "$SERVICE_NAME:2345:respawn:$SERVICE_FILE -d >/dev/null 2>&1 # TSLM 3494 Emulation" >>/etc/inittab
        echo "done"
    fi
    echo -n "Telling init to re-examine /etc/inittab..."
    telinit q
    echo "done"
}


#################################
# MAIN
#################################
whatDist
if [ $DIST -eq 0 ]; then
    echo "detected unsupported OS..."
    exit 1
elif [ $DIST -eq 1 ]; then
    echo "detected RHEL5"
    post3494_RHEL5
elif [ $DIST -eq 2 ]; then
    echo "detected RHEL6"
    post3494_RHEL6
elif [ $DIST -eq 3 ]; then
    echo "detected RHEL7"
    post3494_RHEL7
elif [ $DIST -eq 4 ]; then
    echo "detected SLES10"
    post3494_SLES10
elif [ $DIST -eq 5 ]; then
    echo "detected SLES12"
    post3494_SLES12
elif [ $DIST -eq 9 ]; then
    echo "detected AIX"
    post3494_AIX
else
    echo "detected unsupported OS..."
    exit 1
fi



