#!/bin/sh

###################################
# MAIN
###################################
if [ $# -ne 2 ]; then
    echo "Error[unPostHDM.sh]: - invalid parameter"
    exit 1
fi
SERVICE_NAME=$1
SERVICE_PATH=$2
SERVICE_FILE=$SERVICE_PATH/bin/ermmGenHDM

#
# whatDist()
# DIST value:
# 0: not supported OS
# 1: RHEL5
# 2: RHEL6
# 3: RHEL7
# 4: SLES10-11
# 5: SLES12
# 9: AIX
whatDist()
{
    OS=`uname`
    if [ $OS = "AIX" ]; then # AIX
	DIST=9
    elif [ $OS = "Linux" ]; then # Linux
	if [ -e "/etc/redhat-release" ]; then           # RHEL
	    RHELVER6="`cat /etc/redhat-release | sed -e 's/^.*\(6\.\).*$/\1/'`"
	    RHELVER7="`cat /etc/redhat-release | sed -e 's/^.*\(7\.\).*$/\1/'`"
	    if [ "$RHELVER6" = "6." ]; then #RHEL6
		DIST=2
	    elif [ "$RHELVER7" = "7." ]; then #RHEL7
		DIST=3
	    else                         #RHEL5
		DIST=1
	    fi
	elif [ -e "/etc/SuSE-release" ]; then           # SuSE
	    SLESVER="`cat /etc/SuSE-release | grep VERSION | tr -d ' ' | cut -f2 -d'='`"
	    if [ "$SLESVER" = "12" ]; then
		DIST=5                 #SuSE12
	    else
		DIST=4                 #SuSE10-11
	    fi
	else                                            # Others 
	    DIST=0
	fi
    else
	DIST=0
    fi
}

#
# un post HDM for RHEL5
#
unPostHDM_RHEL5()
{
    if (grep "^$SERVICE_NAME:" /etc/inittab > /dev/null); then
	echo -n "Removing $SERVICE_NAME from /etc/inittab..."
	cp /etc/inittab /etc/initab.hdmsav
	grep -v $SERVICE_FILE /etc/inittab >/tmp/ermminittab
	cp /tmp/ermminittab /etc/inittab
	rm /tmp/ermminittab
        echo "done"
        echo -n "Telling init to re-examine /etc/inittab..."
        telinit q
        echo "done"
    fi
}

#
# un post HDM for RHEL6
#
unPostHDM_RHEL6()
{
    echo -n "Removing $SERVICE_NAME.conf from /etc/init..."
    if [ -e "/etc/init/$SERVICE_NAME.conf" ]; then
	initctl stop $SERVICE_NAME
	rm -f /etc/init/$SERVICE_NAME.conf
    fi
    echo "done"
}

#
# un post HDM for RHEL7
#
unPostHDM_RHEL7()
{
    echo -n "Removing $SERVICE_NAME.service from /etc/systemd/system..."
    if [ -e "/etc/systemd/system/$SERVICE_NAME.service" ]; then
	/usr/bin/systemctl stop $SERVICE_NAME
	/usr/bin/systemctl disable $SERVICE_NAME
	rm /etc/systemd/system/$SERVICE_NAME.service
    fi
    echo "done"
}

#
# un post HDM for SuSE10
#
unPostHDM_SLES10()
{
    if (grep "^$SERVICE_NAME:" /etc/inittab > /dev/null); then
	echo -n "Removing $SERVICE_NAME from /etc/inittab..."
	cp /etc/inittab /etc/initab.hdmsav
	grep -v $SERVICE_FILE /etc/inittab >/tmp/ermminittab
	cp /tmp/ermminittab /etc/inittab
	rm /tmp/ermminittab
        echo "done"
        echo -n "Telling init to re-examine /etc/inittab..."
        telinit q
        echo "done"
    fi
}

#
# un post HDM for SuSE12
#
unPostHDM_SLES12()
{
    echo -n "Removing $SERVICE_NAME.service from /etc/systemd/system..."
    if [ -e "/etc/systemd/system/$SERVICE_NAME.service" ]; then
	/usr/bin/systemctl stop $SERVICE_NAME
	/usr/bin/systemctl disable $SERVICE_NAME
	rm /etc/systemd/system/$SERVICE_NAME.service
    fi
    echo "done"
}

#
# un post HDM for AIX
#
unPostHDM_AIX()
{
    if (grep "^$SERVICE_NAME:" /etc/inittab > /dev/null); then
	echo -n "Removing $SERVICE_NAME from /etc/inittab..."
	cp /etc/inittab /etc/initab.hdmsav
	grep -v $SERVICE_FILE /etc/inittab >/tmp/ermminittab
	cp /tmp/ermminittab /etc/inittab
	rm /tmp/ermminittab
        echo "done"
        echo -n "Telling init to re-examine /etc/inittab..."
        telinit q
        echo "done"
    fi
}

#################################
# MAIN
#################################
whatDist
if [ $DIST -eq 0 ]; then
    echo "detected unsupported OS..."
    exit 1
elif [ $DIST -eq 1 ]; then
    echo "detected RHEL5"
    unPostHDM_RHEL5
elif [ $DIST -eq 2 ]; then
    echo "detected RHEL6"
    unPostHDM_RHEL6
elif [ $DIST -eq 3 ]; then
    echo "detected RHEL7"
    unPostHDM_RHEL7
elif [ $DIST -eq 4 ]; then
    echo "detected SLES10"
    unPostHDM_SLES10
elif [ $DIST -eq 5 ]; then
    echo "detected SLES12"
    unPostHDM_SLES12
elif [ $DIST -eq 9 ]; then
    echo "detected AIX"
    unPostHDM_AIX
else
    echo "detected unsupported OS..."
    exit 1
fi

exit 0

