﻿#!/usr/bin/perl
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2012 All Rights Reserved
#-----------------------------------------------------------------------
#
# @version 1.4 - 18/05/14 23:50:25
#
# @brief
#
# Change activities:
#     - 2004-03-16: Initial version
# Change History:
# 2018/05/15 84766 Confirmation msg should be required for rmlib cmd
#
#-----------------------------------------------------------------------

package Ermm::CLI::RmLib;
use Getopt::Long;
use Term::ReadLine;
use Ermm::CLI::Conf;
use Ermm::CLI::Main;

$synopsis = "rmlib [-v | -help, -? | -match <c>] [<libname>]";

#
# remove library
#
sub rmlib {
    my ($i, $regex, $result);
    my ($response, $match, $library);
    my $help        = 0;
    my $sendString  = "";
    my $matchString = "";

    #
    # process command line options
    #
    $result = GetOptions ("v"        => \$Ermm::CLI::Conf::verbose, # flag
                          "help|?"   => \$help,   # flag
                          "match=s"  => \$match); # string

    #
    # check if we should print usage infos
    #
    if ($result != 1 || $help == 1) {
        Ermm::CLI::Main::printtext('_USAGE_RMLIB', $synopsis);
        exit 1;
    }

    #
    # add ai names to match clause
    #
    foreach $i (1 .. $#ARGV) {
        $matchString .= "streq(Library.\"LibraryName\" \"$ARGV[$i]\")";
    }

    if ($#ARGV > 0) {
        $matchString = "or($matchString)";
    }

    #
    # add match clause if specified
    #
    if ($match) {
        $matchString = "and($matchString $match)";
    }

    if (length($matchString) > 0) {
        $matchString = "match[$matchString]";
    }
    else {
        Ermm::CLI::Main::printtext('EMM7010E');
        Ermm::CLI::Main::printtext('_USAGE_RMLIB', $synopsis);
        exit 1;
    }

    #
    # build send string
    #
    $sendString = "delete task[\"$Ermm::CLI::Conf::taskId\"] type[Library] ";

    $sendString .= "$matchString;\n";

    #
    # confirmation
    #
    my $flag="";
    do{
	Ermm::CLI::Main::printtext('CLICONFIRM');
	$flag=lc(<STDIN>);
	chomp $flag;
	if ($flag eq "no") { exit 1; }
    } until ($flag eq "yes");

    #
    # connect to TSLM server
    #
    Ermm::CLI::Main::connect();

    #
    # send command to server
    #
    Ermm::CLI::Main::send($sendString);

    #
    # receive response from server
    #
    $response = Ermm::CLI::Main::getResponse();

}

1;
