#!/bin/sh
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use,  duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#-----------------------------------------------------------------------
#
# @version 1.1 - 14/08/28 22:02:53
#
# @brief Tool to collect extra information relevant for PD work
# ermmCollectLogsExLinux.sh
# supported only linux
#
#-----------------------------------------------------------------------
basename=tslm-`date +"%y%m%d5/29/18M%S"`
rootdir=/tmp/${basename}
rundir=/opt/IBM/ermm/run

#
# helper routine
#
eval_core()
{
    cmd="$1 >> ${rootdir}/core_env.log 2>&1"
    echo "" >> ${rootdir}/core_env.log 2>&1
    echo "###############################################" >> ${rootdir}/core_env.log 2>&1
    echo "# exec $1" >> ${rootdir}/core_env.log 2>&1
    echo "###############################################" >> ${rootdir}/core_env.log 2>&1
    eval ${cmd}
}

eval_linux()
{
    cmd="$1 >> ${rootdir}/linux_env.log 2>&1"
    echo "" >> ${rootdir}/linux_env.log 2>&1
    echo "###############################################" >> ${rootdir}/linux_env.log 2>&1
    echo "# exec $1" >> ${rootdir}/linux_env.log 2>&1
    echo "###############################################" >> ${rootdir}/linux_env.log 2>&1
    eval ${cmd}
}

eval_db2()
{
    cmd="$1 >> ${rootdir}/db2_env.log 2>&1"
    echo "" >> ${rootdir}/db2_env.log 2>&1
    echo "###############################################" >> ${rootdir}/db2_env.log 2>&1
    echo "# exec $1" >> ${rootdir}/db2_env.log 2>&1
    echo "###############################################" >> ${rootdir}/db2_env.log 2>&1
    eval ${cmd}
}

cp_varlog()
{
    mkdir -p ${rootdir}/varlog
    echo "copy /var/log/$1 to ${rootdir}/varlog" >> ${rootdir}/linux_env.log 2>&1
    eval "cp -fp /var/log/$1 ${rootdir}/varlog >> ${rootdir}/linux_env.log 2>&1"
}

cp_core()
{
    mkdir -p ${rootdir}/core
    for i in `ls ${rundir}`;
      do
      if [ -d ${rundir}/$i ]; then
	  for j in `ls -t ${rundir}/$i/core* 2>/dev/null`;
	    do
	    echo "copy $j to ${rootdir}/core/$i" >> ${rootdir}/core_env.log 2>&1
	    mkdir -p ${rootdir}/core/$i
	    cp -fp $j ${rootdir}/core/$i
	    break
	  done
      fi
    done
}

#
# check core environment
#
chk_core_env()
{
    echo "check core environment"
    eval_core "ulimit -a"    
    eval_core "cat /home/ermm/.bashrc"
    eval_core "cat /etc/security/limits.conf"
    eval_core "cat /etc/security/limits.d/*.conf"
    cp_core
    return 0
}

#
# check linux environment
#
chk_linux_env()
{
    echo "check linux environment"
    eval_linux "cat /etc/issue"    
    eval_linux "sudo -l"
    eval_linux "set"
    eval_linux "sudo -u ermm sh -c \"cd /home/ermm;pwd;. .bashrc;set\""
    eval_linux "ps -ef"
    eval_linux "rpm -qa"
    eval_linux "ls -l /boot"
    eval_linux "df -h"
    eval_linux "vmstat"
    eval_linux "dmidecode"
    eval_linux "cat /proc/cpuinfo"
    eval_linux "lspci"
    eval_linux "cat /proc/scsi/IBMtape"
    eval_linux "cat /proc/scsi/IBMchanger"
    cp_varlog "messages*"
    cp_varlog "*_atl.log"
    cp_varlog "lin_tape.*"
    cp_varlog "ibmscs.*"
    return 0
}

#
# check db2 environment
#
chk_db2_env()
{
    echo "check db2 environment"
    echo "check unprintable character"
    eval_db2 "sudo -u ermm sh -c \"cd /home/ermm;pwd;. .bashrc;db2 connect to ermm;db2 select \* from m_slot where TRANSLATE\(UPPER\(slotname\),\'\',\'ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890\!@#$%^\&*\(\)-=+/\\{}[]\;:.,<>? \'\) \<\> \'\';db2 connect reset\""
    eval_db2 "sudo -u ermm sh -c \"cd /home/ermm;pwd;. .bashrc;db2 connect to ermm;db2 select \* from m_drive where TRANSLATE\(UPPER\(drivename\),\'\',\'ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890\!@#$%^\&*\(\)-=+/\\{}[]\;:.,<>? \'\) \<\> \'\';db2 connect reset\""
    eval_db2 "sudo -u ermm sh -c \"cd /home/ermm;pwd;. .bashrc;db2 connect to ermm;db2 select \* from m_cartridge where TRANSLATE\(UPPER\(cartridgepcl\),\'\',\'ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890\!@#$%^\&*\(\)-=+/\\{}[]\;:.,<>? \'\) \<\> \'\';db2 connect reset\""
    eval_db2 "sudo -u ermm sh -c \"cd /home/ermm;pwd;. .bashrc;db2 connect to ermm;db2 select \* from m_volume where TRANSLATE\(UPPER\(volumename\),\'\',\'ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890\!@#$%^\&*\(\)-=+/\\{}[]\;:.,<>? \'\) \<\> \'\';db2 connect reset\""

    return 0
}

#
# capture dbs contents
#
capture_db2_contents()
{
    echo "capture db2 contents"
    mkdir -p ${rootdir}/data
    chown ermm.ermm ${rootdir}/data
    pushd ${rootdir}
    eval_db2 "sudo -u ermm sh -c \". /home/ermm/.bashrc;/opt/IBM/ermm/tools/export_from_tslm\""
    popd
    return 0
}

#
# main
#
if [ "${USER}" != "root" ]; then
  echo ermmCollectLogsExLinux.sh must be executed via root
  exit 1
fi

echo "IBM Tape System Library Manager"
echo "Licensed Materials - Property of IBM"
echo "(C) Copyright IBM Corp. 2004, 2014 All Rights Reserved"
echo "US Government Users Restricted Rights - Use, duplication or"
echo "disclosure restricted by GSA ADP Schedule Contract with IBM Corp."


# make root dir
mkdir -p ${rootdir}
chk_core_env
chk_linux_env
chk_db2_env
capture_db2_contents
pushd ${rootdir}
cd ..
tar czvf ${basename}.tar.gz ${basename}
popd
echo "/tmp/${basename}.tar.gz was created"




