#!/usr/bin/perl -w
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use,  duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#-----------------------------------------------------------------------
#
# @version 1.2 - 13/06/21 01:04:45
#
# Sample scripts for TSLM
#
#-----------------------------------------------------------------------
my $volumeList = "./volumeList_Kanda.txt";
my $TSLMBatch = "./TSLMBatch.sh";
my $TSMApp = "TSM";
my $TSMInstance = "TSMINST";
my $volLen = 8;
my $PCLSuffix = "";
my $LIB = "TSLM";

my $counter = 0;
my $counterScratch = 0;
my $counterCheck = 0;
my $verbose = 0;


# Read the volume lists
# Format of the list:
# D00001,Private
# D00004,Scratch
# D00001,Private

#####################
# Library
#####################
if (!open(FILE, $volumeList)) {
	print "Can not open file $volumeList\n";
	return 0;
} 
if (!open($OUTFILE, ">$TSLMBatch")) {
	print "Can not open file $OUTFILE\n";
	return 0;
}

print $OUTFILE "#!/usr/bin/ksh\n";

print "\nCreate $LIB alloc commands for volumes in library $LIB\n";
while (<FILE>) {
	if ($_ =~ /.*Scratch.*/) {
		if ($verbose == 1) {
			print "Skipping Scratch: $_";
		}
		$counterScratch++;
	} elsif ($_ =~ /^(\S{$volLen}),(\S+)/) {
	 	$TSLMCmd = "ermmtool -c $TSMApp -i $TSMInstance alloc $1 -pcl $1$PCLSuffix";	
		if ($verbose == 1) {
			print "$TSLMCmd\n";
		}
		print $OUTFILE "echo \"Running: $TSLMCmd\"\n";
		print $OUTFILE "$TSLMCmd\n";
		$counter++;
	} else {
		print "Neither scratch nor private. Skipping: $_";
		$counterCheck++;
	}
}
if (!close(FILE)) {
	print "Can not close file $volumeList\n";
}
if (!close($OUTFILE)) {
	print "Can not close file $TSLMBatch\n";
}
print "Number of data volumes found for : $counter\n";
print "Number of scratch volumes for : $counterScratch\n";
print "Created batch file: $TSLMBatch\n";
if ($counterCheck != 0) {
	print "Skipped $counterCheck line(s) that was neither scratch nor private. Check output !\n";
} else {
	print "OK. Done for Library $LIB\n";
}


